using System;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;
using System.Diagnostics;
using System.IO;

namespace QD_GUI
{
    public class QueueDumpUtil
    {
        // qd released 14 November 2006 (fr 049)
        private string sDate = "";                  // 14 November 2006
        private string sFunctionalRevision = "";    // 049
        private string sPath = "";                  // C:\Program Files\QD-GUI\qd.exe
        private string sInfoFile = "";              // parameter: -n (C:\Program Files\QD-GUI\qdinfo.dat)
        private string sFahDirectory = "";          // parameter: -f (C:\Program Files\Folding@Home)
        private bool bCompactOutput = false;        // parameter: -c
        private bool bOnlyParseQueue = false;       // parameter: -p
        private string sSystemType = "";            // parameter: -s (linux, windows, mac)
        private string sOverrideTimeZone = "";      // parameter: -t (+HHMM or -HHMM)
        private bool bForceUTC = false;             // parameter: -z (same as -t +0000)
        private bool bPrintVersion = false;         // parameter: -v
        private bool bValidXMLOutput = true;        // parameter: -X
        private string sOutput = "";
        private Dictionary<string, string> oArguments = new Dictionary<string, string>();

        #region Constructors

        public QueueDumpUtil()
        {
            try
            {
                sPath = Properties.Settings.Default.qd_FileName;

                oArguments = GetArguments();
            }
            catch(Exception oException)
            {
                throw new Exception(oException.Message);
            }
        }

        #endregion Constructors

        #region Properties

        public string Date
        {
            get
            {
                return sDate;
            }
            set
            {
                sDate = value;
            }
        }

        public string FunctionalRevision
        {
            get
            {
                return sFunctionalRevision;
            }
            set
            {
                sFunctionalRevision = value;
            }
        }

        public string Path
        {
            get
            {
                return sPath;
            }
            set
            {
                sPath = value;
            }
        }

        public string InfoFile
        {
            get
            {
                return sInfoFile;
            }
            set
            {
                sInfoFile = value;
            }
        }

        public string FahDirectory
        {
            get
            {
                return sFahDirectory;
            }
            set
            {
                sFahDirectory = value;
            }
        }

        public bool CompactOutput
        {
            get
            {
                return bCompactOutput;
            }
            set
            {
                bCompactOutput = value;
            }
        }

        public bool OnlyParseQueue
        {
            get
            {
                return bOnlyParseQueue;
            }
            set
            {
                bOnlyParseQueue = value;
            }
        }

        public string SystemType
        {
            get
            {
                return sSystemType;
            }
            set
            {
                sSystemType = value;
            }
        }

        public string OverrideTimeZone
        {
            get
            {
                return sOverrideTimeZone;
            }
            set
            {
                sOverrideTimeZone = value;
            }
        }

        public bool ForceUTC
        {
            get
            {
                return bForceUTC;
            }
            set
            {
                bForceUTC = value;
            }
        }

        public bool PrintVersion
        {
            get
            {
                return bPrintVersion;
            }
            set
            {
                bPrintVersion = value;
            }
        }

        public bool ValidXMLOutput
        {
            get
            {
                return bValidXMLOutput;
            }
            set
            {
                bValidXMLOutput = value;
            }
        }

        #endregion Properties

        #region Static Methods

        public static Dictionary<string, string> GetArguments()
        {
            Dictionary<string, string> roArguments = new Dictionary<string, string>();

            //qd.exe" --help
            //
            //Usage: My Documents\tmp\folding\qd.exe [<flags> <arguments required by flags>]*
            // -u      Print this usage message (and exit)
            // -h      Print an explanation of the status codes (and exit)
            // -i      Format with deeper indentation
            // -c      Compact format (omit several lines)
            // -p      Don't look at FAH files other than <queue.dat>
            if (Properties.Settings.Default.OnlyParseQueue == true)
            {
                roArguments.Add("-p", "");
            }
            // -P      Print extra Project, Run, Clone, Generation line like in FAHlog.txt
            // -f dir  Explicitly specify folding directory
            if(!String.IsNullOrEmpty(Properties.Settings.Default.FahDirectory))
            {
                roArguments.Add("-f", Properties.Settings.Default.FahDirectory);
            }
            // -q file Explicitly specify queue data file (implies -p)
            // -n file Explicitly specify qd info file
            if (!String.IsNullOrEmpty(Properties.Settings.Default.qdinfo_FileName))
            {
                roArguments.Add("-n", Properties.Settings.Default.qdinfo_FileName);
            }
            // -m file Explicitly specify EM III info file
            // -s {linux, windows, mac}  Specify type of queue data file
            if (!String.IsNullOrEmpty(Properties.Settings.Default.SystemType)) 
            {
                if (  Properties.Settings.Default.SystemType == "linux"
                   || Properties.Settings.Default.SystemType == "windows"
                   || Properties.Settings.Default.SystemType == "mac"
                   )
                {
                    roArguments.Add("-s", Properties.Settings.Default.SystemType);
                }                
            }
            // -t zone Override time zone (+HHMM or -HHMM)
            if(!String.IsNullOrEmpty(Properties.Settings.Default.OverrideTimeZone))
            {
                roArguments.Add("-t", Properties.Settings.Default.OverrideTimeZone);
            }
            // -z      Force UTC (same as -t +0000)
            if (Properties.Settings.Default.ForceUTC == true)
            {
                roArguments.Add("-z", "");
            }
            // -v      Just print version information and stop
            // -d      Print debug dump
            // -e      Print all entries even if they're garbage
            // -l      Format for log entry data
            // -L      Format for log entry data with specified ID string
            // -x      Print output in Well Formed XML
            // -X      Print output in Valid XML with Inline DTD
            roArguments.Add("-X", "");

            return roArguments;
        }

        public static FunctionalRevision GetFunctionalRevision()
        {
            string sFunctionalRevision = "";
            string sFunctionalRevisionAvailable = "";
            bool bNewerFunctionalRevisionAvailable = false;

            Debug.WriteLine("Running qd.exe -n <qdinfo.dat> -v -X");

            Dictionary<string, string> oMyArgument = new Dictionary<string, string>();
            oMyArgument.Add("-n", Properties.Settings.Default.qdinfo_FileName);
            //oMyArgument.Add("-f", Properties.Settings.Default.FahDirectory);
            oMyArgument.Add("-v", "");
            oMyArgument.Add("-X", "");

            // Run qd.exe to get its Functional Revision
            QueueDumpUtil oQueueDumpUtil = new QueueDumpUtil();
            string sOutput = oQueueDumpUtil.RunWithArguments(oMyArgument);

            Stream oXmlStream = new MemoryStream(System.Text.ASCIIEncoding.ASCII.GetBytes(sOutput));

            QueueDump oQueueDump = new QueueDump();
            oQueueDump = XmlQueueDump.ParseXmlStream(oXmlStream, oQueueDump);
            sFunctionalRevision = oQueueDump.QD.FunctionalRevision;
           
            // Parse qdinfo.dat to get the latest available Functional Revision
            //sFunctionalRevisionAvailable = "090";
            QDInfo oQDInfo = new QDInfo();
            oQDInfo = QDInfo.ParseFile(Properties.Settings.Default.qdinfo_FileName, oQDInfo);
            sFunctionalRevisionAvailable = oQDInfo.FunctionalRevision;

            int iFunctionalRevision = 0;
            int iFunctionalRevisionAvailable = Convert.ToInt32(sFunctionalRevisionAvailable);

            // fr 067
            // fr EXPERIMENTAL
            string sRegex = @"^\d+$";
            if (Regex.IsMatch(sFunctionalRevision, sRegex))
            {
                // Experimental releases of qd won't be updated
                iFunctionalRevision = Convert.ToInt32(sFunctionalRevision);
                if (iFunctionalRevision < iFunctionalRevisionAvailable)
                {
                    bNewerFunctionalRevisionAvailable = true;
                }
            }

            FunctionalRevision oFunctionalRevision = new FunctionalRevision();
            oFunctionalRevision.CurrentFunctionalRevision = iFunctionalRevision;
            oFunctionalRevision.CurrentFunctionalRevisionString = sFunctionalRevision;
            oFunctionalRevision.FunctionalRevisionAvailable = iFunctionalRevisionAvailable;
            oFunctionalRevision.FunctionalRevisionAvailableString = sFunctionalRevisionAvailable;
            oFunctionalRevision.NewerFunctionalRevisionAvailable = bNewerFunctionalRevisionAvailable;

            return oFunctionalRevision;
        }
        
        #endregion Static Methods

        #region Public Methods

        public string Run()
        {
            // Run qd with the default arguments.
            // The default arguments are configured through the OptionsWindow,
            // and stored in the Settings.settings file.
            // The arguments object is initialized by the Constructor

            return RunWithArguments(oArguments);
        }

        public string RunWithArguments(Dictionary<string, string> voArguments)
        {
            StringBuilder oArgumentsString = new StringBuilder();

            #region Construct Argument String

            foreach (string sKey in voArguments.Keys)
            {
                switch (sKey)
                {
                    //qd.exe" --help
                    //
                    //Usage: My Documents\tmp\folding\qd.exe [<flags> <arguments required by flags>]*
                    // -u      Print this usage message (and exit)
                    // -h      Print an explanation of the status codes (and exit)
                    // -i      Format with deeper indentation
                    // -c      Compact format (omit several lines)
                    // -p      Don't look at FAH files other than <queue.dat>
                    case "-p":
                        oArgumentsString.AppendFormat(" {0}", sKey);
                        break;
                    // -P      Print extra Project, Run, Clone, Generation line like in FAHlog.txt
                    // -f dir  Explicitly specify folding directory
                    case "-f":
                        // Folding@Home client directory
                        oArgumentsString.AppendFormat(" {0} \"{1}\"", sKey, voArguments[sKey]);
                        break;
                    // -q file Explicitly specify queue data file (implies -p)
                    // -n file Explicitly specify qd info file
                    case "-n":
                        oArgumentsString.AppendFormat(" {0} \"{1}\"", sKey, voArguments[sKey]);
                        break;
                    // -m file Explicitly specify EM III info file
                    // -s {linux, windows, mac}  Specify type of queue data file
                    case "-s":
                        oArgumentsString.AppendFormat(" {0} \"{1}\"", sKey, voArguments[sKey]);
                        break;
                    // -t zone Override time zone (+HHMM or -HHMM)
                    case "-t":
                        oArgumentsString.AppendFormat(" {0} \"{1}\"", sKey, voArguments[sKey]);
                        break;
                    // -z      Force UTC (same as -t +0000)
                    case "-z":
                        oArgumentsString.AppendFormat(" {0}", sKey);
                        break;
                    // -v      Just print version information and stop
                    // -d      Print debug dump
                    // -e      Print all entries even if they're garbage
                    // -l      Format for log entry data
                    // -L      Format for log entry data with specified ID string
                    // -x      Print output in Well Formed XML
                    // -X      Print output in Valid XML with Inline DTD
                    case "-X":
                        oArgumentsString.AppendFormat(" {0}", sKey);
                        break;
                }
            }

            #endregion Construct Argument String

            // Start the child process.
            Process oProcess = new System.Diagnostics.Process();

            // Redirect the output stream of the child process.
            oProcess.StartInfo.UseShellExecute = false;
            oProcess.StartInfo.CreateNoWindow = true;
            oProcess.StartInfo.RedirectStandardOutput = true;
            oProcess.StartInfo.FileName = Properties.Settings.Default.qd_FileName;
            oProcess.StartInfo.Arguments = oArgumentsString.ToString();
            oProcess.Start();

            // Read the output stream first and then wait.
            sOutput = oProcess.StandardOutput.ReadToEnd();
            oProcess.WaitForExit();

            return sOutput;
        }

        public void SaveXmlToFile(string vsFile)
        {
            //////////////////////////////////////////////
            // Write the qd output to a Temp file on disk.

            // Specify file, instructions, and privelegdes
            FileStream oFileSteam = new FileStream(vsFile, FileMode.Create, FileAccess.Write);

            // Create a new stream to write to the file
            StreamWriter sw = new StreamWriter(oFileSteam);

            // Write a string to the file
            sw.Write(sOutput);

            // Close StreamWriter
            sw.Close();

            // Close file
            oFileSteam.Close();
        }

        #endregion Public Methods
    }
}
