﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Net;
using System.IO;
using System.Threading;

namespace QD_GUI
{
    public partial class WebUpdateWindow : Form
    {
        private Dictionary<string, WebUpdateItem> oWebUpdateItems = new Dictionary<string,WebUpdateItem>();
        private string sQDInfoStatus = "";
        private string sQDInfoText = "";
        private string sQDStatus = "";
        private string sQDText = "";
        private string sStatus = "";
        private string sDownloadStatus = "";
        private StringBuilder oDownloadStatus = new StringBuilder("");
        //private int iSecond = 1000; // There are 1000 milliseconds in 1 second
        private int iHalfSecond = 500;
        
        #region Constructors

        public WebUpdateWindow()
        {
            InitializeComponent();

            // Build the WebUpdateItems list
            foreach (string sName in Properties.Settings.Default.WebUpdateItems)
            {
                WebUpdateItem oItem = new WebUpdateItem(sName);
                switch(sName)
                {
                    case "qd":
                        oItem.Url = Properties.Settings.Default.qd_Url;
                        oItem.FileName = Properties.Settings.Default.qd_FileName;
                        oItem.LastModified = Properties.Settings.Default.qd_LastModified;
                        oItem.LastUpdate = Properties.Settings.Default.qd_LastUpdate;

                        // Add the WebUpdateItem to the list
                        oWebUpdateItems.Add(sName, oItem);
                        break;
                    case "qdinfo":
                        oItem.Url = Properties.Settings.Default.qdinfo_Url;
                        oItem.FileName = Properties.Settings.Default.qdinfo_FileName;
                        oItem.LastModified = Properties.Settings.Default.qdinfo_LastModified;
                        oItem.LastUpdate = Properties.Settings.Default.qdinfo_LastUpdate;

                        // Add the WebUpdateItem to the list
                        oWebUpdateItems.Add(sName, oItem);
                        break;
                }
            }

            // Get the default values from the status labels
            this.sQDInfoStatus = lblWebUpdateItemQDInfoStatus.Text;
            this.sQDInfoText = lblWebUpdateItemQDInfoText.Text;
            this.sQDStatus = lblWebUpdateItemQDStatus.Text;
            this.sQDText = lblWebUpdateItemQDText.Text;
            this.sStatus = lblStatusValue.Text;
            this.sDownloadStatus = lblDownloadStatus.Text;
        }

        #endregion Constructors

        private void ResetLabels()
        {
            // Reset the DownloadStatus string to its default value
            oDownloadStatus.Remove(0, oDownloadStatus.Length);

            // Reset the labels to their default values
            lblWebUpdateItemQDInfoStatus.Text = sQDInfoStatus;
            lblWebUpdateItemQDInfoText.Text = sQDInfoText;
            lblWebUpdateItemQDStatus.Text = sQDStatus;
            lblWebUpdateItemQDText.Text = sQDText;
            lblStatusValue.Text = sStatus;
            lblDownloadStatus.Text = sDownloadStatus;

            this.Update();
        }

        public void WebUpdate()
        {
            // Download qdinfo.dat first
            Download("qdinfo");

            oDownloadStatus.AppendFormat("Checking for newer Functional Revision of qd.exe.\r\n");
            lblDownloadStatus.Text = oDownloadStatus.ToString();
            this.Update();

            FunctionalRevision oFunctionalRevision = QueueDumpUtil.GetFunctionalRevision();

            oDownloadStatus.AppendFormat("Current Functional Revision: {0}\r\n", oFunctionalRevision.CurrentFunctionalRevisionString);
            lblDownloadStatus.Text = oDownloadStatus.ToString();
            this.Update();

            if (oFunctionalRevision.NewerFunctionalRevisionAvailable == true)
            {
                oDownloadStatus.AppendFormat("Newer Functional Revision of qd.exe is available: {0}.\r\n", oFunctionalRevision.FunctionalRevisionAvailableString);
                
                Download("qd");
            }
            else
            {
                oDownloadStatus.AppendFormat("No newer Functional Revision of qd.exe is available: {0}.\r\n\r\n", oFunctionalRevision.FunctionalRevisionAvailableString);
            }
            lblDownloadStatus.Text = oDownloadStatus.ToString();
            this.Update();
            Thread.Sleep(iHalfSecond);

            oDownloadStatus.AppendFormat("Update completed.\r\n");
            lblDownloadStatus.Text = oDownloadStatus.ToString();

            lblStatusValue.Text = "Idle";
            this.Update();
            btnCheckNow.Focus();
        }

        /// <summary>
        /// Download both qd.exe and qdinfo.dat
        /// </summary>
        private void Download()
        {
            Download("all");
        }

        /// <summary>
        /// Download only the given file.
        /// </summary>
        /// <param name="vsFile">"qd", "qdinfo" or "all"</param>
        private void Download(string vsFile)
        {
            string sLastModified = "";
            List<string> saDownloadFiles = new List<string>();

            switch (vsFile)
            {
                case "qdinfo":
                    saDownloadFiles.Add(vsFile);
                    break;
                case "qd":
                    saDownloadFiles.Add(vsFile);
                    break;
                case "all":
                    saDownloadFiles.Add("qdinfo");
                    saDownloadFiles.Add("qd");
                    break;
            }

            for (int i = 0; i < saDownloadFiles.Count; i++) {
                if (saDownloadFiles[i].ToString() == "qdinfo" && !String.IsNullOrEmpty(Properties.Settings.Default.qdinfo_LastModified))
                {
                    sLastModified = Properties.Settings.Default.qdinfo_LastModified;
                }
                if (saDownloadFiles[i].ToString() == "qd" && !String.IsNullOrEmpty(Properties.Settings.Default.qd_LastModified))
                {
                    sLastModified = Properties.Settings.Default.qd_LastModified;
                }

                DownloadFile(saDownloadFiles[i].ToString(), sLastModified);
                Thread.Sleep(iHalfSecond);
            }
        }

        private void DownloadFile(string vsFile, string vsLastModified)
        {
            if(oWebUpdateItems[vsFile] == null) 
            {
                throw new Exception("No WebUpdateItem for: " + vsFile);
            }
            if(String.IsNullOrEmpty(oWebUpdateItems[vsFile].Url))
            {
                throw new Exception("No Url settings for WebUpdateItem: " + vsFile);
            }
            if(String.IsNullOrEmpty(oWebUpdateItems[vsFile].FileName))
            {
                throw new Exception("No FileName settings for WebUpdateItem: " + vsFile);
            }

            // Check if we can write to the file
            try
            {
                FileStream oFileStream = File.OpenWrite(oWebUpdateItems[vsFile].FileName);
                oFileStream.Close();
            }
            catch
            {
                throw new Exception("Cannot write to file: " + oWebUpdateItems[vsFile].FileName);
            }

            // Mark which file will be checked now
            lblStatusValue.Text = "Checking for updates";
            oDownloadStatus.AppendFormat("Preparing to download: {0}\r\n", oWebUpdateItems[vsFile].Url);
            lblDownloadStatus.Text = oDownloadStatus.ToString();
            this.Update();

            switch (vsFile)
            {
                case "qdinfo":
                    lblWebUpdateItemQDInfoStatus.Text = "+";
                    break;
                case "qd":
                    lblWebUpdateItemQDStatus.Text = "+";
                    break;
            }
            this.Update();
            Thread.Sleep(iHalfSecond);
            
            // Try to download the file
            try
            {
                switch (vsFile)
                {
                    case "qd":
                        oDownloadStatus.AppendFormat("Downloading qd.exe ...");
                        lblDownloadStatus.Text = oDownloadStatus.ToString();
                        break;
                    case "qdinfo":
                        oDownloadStatus.AppendFormat("Downloading qdinfo.dat ...");
                        lblDownloadStatus.Text = oDownloadStatus.ToString();
                        break;
                }
                this.Update();
                Thread.Sleep(iHalfSecond);

                Uri oUri = new Uri(oWebUpdateItems[vsFile].Url);

                WebClient oWebClient = new WebClient();
                oWebClient.Headers.Add(HttpRequestHeader.UserAgent, "QD-GUI.WebUpdate");
                oWebClient.DownloadFile(oUri, oWebUpdateItems[vsFile].FileName);

                oDownloadStatus.AppendFormat(" done.\r\n");
                lblDownloadStatus.Text = oDownloadStatus.ToString();
                this.Update();
                Thread.Sleep(iHalfSecond);
                
                //Properties.Settings oSettings = new Properties.Settings();
                string sLastModified = oWebClient.ResponseHeaders[HttpResponseHeader.LastModified];
                StringBuilder oString = new StringBuilder();
                
                switch(vsFile)
                {
                    case "qd":
                        Properties.Settings.Default.qd_LastModified = sLastModified;
                        Properties.Settings.Default.qd_LastUpdate = DateTime.Now.ToString("s");
                        Properties.Settings.Default.Save();

                        oString.Remove(0, oString.Length);        
                        oString.AppendFormat("Saved qd.exe\r\n");
                        oString.AppendFormat("Last Modified: {0}\r\n", Properties.Settings.Default.qd_LastModified);
                        oString.AppendFormat("Last Update: {0}\r\n", Properties.Settings.Default.qd_LastUpdate);

                        oDownloadStatus.AppendFormat("{0}\r\n", oString.ToString());

                        lblDownloadStatus.Text = oDownloadStatus.ToString();

                        break;
                    case "qdinfo":                      
                        Properties.Settings.Default.qdinfo_LastModified = sLastModified;
                        Properties.Settings.Default.qdinfo_LastUpdate = DateTime.Now.ToString("s");
                        Properties.Settings.Default.Save();

                        oString.Remove(0, oString.Length);
                        oString.AppendFormat("Saved qdinfo.dat\r\n");
                        oString.AppendFormat("Last Modified: {0}\r\n", Properties.Settings.Default.qdinfo_LastModified);
                        oString.AppendFormat("Last Update: {0}\r\n", Properties.Settings.Default.qdinfo_LastUpdate);

                        oDownloadStatus.AppendFormat("{0}\r\n", oString.ToString());

                        lblDownloadStatus.Text = oDownloadStatus.ToString();

                        break;
                }
                this.Update();
            }
            catch(Exception oException)
            {
                throw new Exception(oException.Message);
            }
        }

        private void btnCheckNow_Click(object sender, EventArgs e)
        {
            try
            {
                this.Cursor = Cursors.WaitCursor;

                ResetLabels();
                Thread.Sleep(iHalfSecond);
                WebUpdate();

                this.Cursor = Cursors.Default;
            }
            catch (Exception oException)
            {
                StringBuilder oString = new StringBuilder();
                oString.Append("Error: WebUpdate failed.\r\n");
                oString.Append("\r\n");
                oString.Append(oException.Message + ".\r\n");

                MessageBox.Show(oString.ToString());
            }
        }
    }
}
